import Chart from 'chart.js';
const { createElement, render, useState,useEffect,Fragment} = wp.element;
function isEmpty(obj){
    if (obj == null) return true;
    if (typeof obj === 'undefined') return true;
    if (typeof obj == 'number') return false;
    if (Array.isArray(obj) || typeof obj === 'string' || obj instanceof String) return obj.length === 0;
    for (var key in obj) if (obj.hasOwnProperty(key)) return false;
    return true;
}

const QuizStatsChart = (props) => {

	const [quiz,setQuiz] = useState(props.quiz);
	const [ready,setReady] = useState(false);
	const [chartRef,setChartRef] = useState('');
	const [currentChart,setCurrentChart] = useState({});

	const [chartData,setChartData] = useState({});


	useEffect(()=>{
		let correct = 0;
		let incorrect = 0;
		let ommitted = 0;
		let count = 0;
		if(props.quiz && props.quiz.meta && props.quiz.meta.questions){
			props.quiz.meta.questions.map((question,i)=>{
				if(question.hasOwnProperty('usercorrect') && question.hasOwnProperty('marked_answer') && !isEmpty(question.marked_answer)){
					if(parseFloat(question.usercorrect) > 0){
						correct++;
					}else{
						incorrect++;
					}
				}else{
					ommitted++;
				}
				count++;
			});
			setChartData({'correct':correct,'incorrect':incorrect,'ommitted':ommitted});
			if(chartRef != ''){
				let data = {
				    datasets: [{
				        data: [correct, incorrect, ommitted],
				        backgroundColor: ['#82c362', "#dc6a6a", "#5381ab"],
				    }],

				    // These labels appear in the legend and in the tooltips when hovering different arcs
				    labels: [
				        window.wplms_quiz_data.translations.correct+' ('+correct+')'+' ('+Math.round(((correct/count)*100))+'%)',
				        window.wplms_quiz_data.translations.incorrect+' ('+incorrect+')'+' ('+Math.round(((incorrect/count)*100))+'%)',
				        window.wplms_quiz_data.translations.unattempted+' ('+ommitted+')'+' ('+Math.round(((ommitted/count)*100))+'%)'
				    ]
				};

				let PiChart = new Chart(chartRef,  {
				    type: 'pie',
				    data: data,
				});
				setCurrentChart(PiChart);
			}
		}
		
	},[chartRef]);

	return (
		
		<div className="quiz_stats_chart">
			
			{
				(quiz && quiz.meta && quiz.meta.questions && quiz.meta.questions.length && quiz.submitted )?
					<Fragment>
					{
						(currentChart && chartData)?
							<div className="quiz_stats_chart_pie">
								<canvas ref={(ref)=>{
									if(ref && chartRef == ''){
										setChartRef(ref);
									}
								}}></canvas>
							</div>
						:''
					}
					<div className="question_stats_content_wrapper">
					<span className="question_stats_content_heading">{window.wplms_quiz_data.translations.historical}</span>
					<div className="question_stats_content">
						{
							quiz.meta.questions.map((question,i)=>{
								if(question.hasOwnProperty('correct_data')){
									return <div className={(question.usercorrect)?'checked_answer correct':'checked_answer incorrect'}>
										<span>{window.wplms_quiz_data.translations.q}{(i+1)}</span>
										<span>{question.correct_data}%</span> 
									</div>
								}
							})
						}

					</div>
					</div>
					</Fragment>
				:''
			}
			
			
		</div>
	  
	);
}
export default QuizStatsChart;